#include <iostream>
#include <vector>
#include <cmath>
#include <algorithm>

class Color {
public:
    unsigned char r, g, b;
    Color(unsigned char r = 0, unsigned char g = 0, unsigned char b = 0) : r(r), g(g), b(b) {}
};

class Image {
private:
    std::vector<std::vector<Color>> pixels;
    int width, height;

public:
    Image(int w, int h) : width(w), height(h), pixels(h, std::vector<Color>(w)) {}

    void setPixel(int x, int y, const Color& c) {
        if (x >= 0 && x < width && y >= 0 && y < height) {
            pixels[y][x] = c;
        }
    }

    Color getPixel(int x, int y) const {
        if (x >= 0 && x < width && y >= 0 && y < height) {
            return pixels[y][x];
        }
        return Color();
    }

    int getWidth() const { return width; }
    int getHeight() const { return height; }
};

class QuadTree {
private:
    struct Node {
        Color color;
        Node* children[4];
        int x, y, size;

        Node(int x, int y, int size) : x(x), y(y), size(size) {
            for (int i = 0; i < 4; ++i) children[i] = nullptr;
        }

        ~Node() {
            for (int i = 0; i < 4; ++i) delete children[i];
        }
    };

    Node* root;
    int threshold;

    bool isLeaf(Node* node) const {
        return node->children[0] == nullptr;
    }

    Color averageColor(const Image& image, int x, int y, int size) const {
        long long r = 0, g = 0, b = 0;
        int count = 0;
        for (int i = y; i < y + size; ++i) {
            for (int j = x; j < x + size; ++j) {
                Color c = image.getPixel(j, i);
                r += c.r; g += c.g; b += c.b;
                ++count;
            }
        }
        return Color(r / count, g / count, b / count);
    }

    int colorDifference(const Color& c1, const Color& c2) const {
        return std::abs(c1.r - c2.r) + std::abs(c1.g - c2.g) + std::abs(c1.b - c2.b);
    }

    void buildTree(Node* node, const Image& image) {
        Color avgColor = averageColor(image, node->x, node->y, node->size);
        node->color = avgColor;

        if (node->size == 1) return;

        bool shouldSplit = false;
        for (int y = node->y; y < node->y + node->size; ++y) {
            for (int x = node->x; x < node->x + node->size; ++x) {
                if (colorDifference(image.getPixel(x, y), avgColor) > threshold) {
                    shouldSplit = true;
                    break;
                }
            }
            if (shouldSplit) break;
        }

        if (shouldSplit) {
            int newSize = node->size / 2;
            node->children[0] = new Node(node->x, node->y, newSize);
            node->children[1] = new Node(node->x + newSize, node->y, newSize);
            node->children[2] = new Node(node->x, node->y + newSize, newSize);
            node->children[3] = new Node(node->x + newSize, node->y + newSize, newSize);

            for (int i = 0; i < 4; ++i) {
                buildTree(node->children[i], image);
            }
        }
    }

    void reconstruct(Node* node, Image& image) const {
        if (isLeaf(node)) {
            for (int y = node->y; y < node->y + node->size; ++y) {
                for (int x = node->x; x < node->x + node->size; ++x) {
                    image.setPixel(x, y, node->color);
                }
            }
        } else {
            for (int i = 0; i < 4; ++i) {
                reconstruct(node->children[i], image);
            }
        }
    }

public:
    QuadTree(const Image& image, int threshold) : threshold(threshold) {
        root = new Node(0, 0, std::max(image.getWidth(), image.getHeight()));
        buildTree(root, image);
    }

    ~QuadTree() { delete root; }

    Image reconstruct() const {
        Image result(root->size, root->size);
        reconstruct(root, result);
        return result;
    }
};

int main() {
    // Create a sample 8x8 image
    Image originalImage(8, 8);
    for (int y = 0; y < 8; ++y) {
        for (int x = 0; x < 8; ++x) {
            if (x < 4 && y < 4) {
                originalImage.setPixel(x, y, Color(255, 0, 0)); // Red quadrant
            } else if (x >= 4 && y < 4) {
                originalImage.setPixel(x, y, Color(0, 255, 0)); // Green quadrant
            } else if (x < 4 && y >= 4) {
                originalImage.setPixel(x, y, Color(0, 0, 255)); // Blue quadrant
            } else {
                originalImage.setPixel(x, y, Color(255, 255, 0)); // Yellow quadrant
            }
        }
    }

    // Create QuadTree with a threshold of 50
    QuadTree qt(originalImage, 50);

    // Reconstruct the image from the QuadTree
    Image reconstructedImage = qt.reconstruct();

    // Print original and reconstructed images
    std::cout << "Original Image:" << std::endl;
    for (int y = 0; y < 8; ++y) {
        for (int x = 0; x < 8; ++x) {
            Color c = originalImage.getPixel(x, y);
            std::cout << "(" << (int)c.r << "," << (int)c.g << "," << (int)c.b << ") ";
        }
        std::cout << std::endl;
    }

    std::cout << "\nReconstructed Image:" << std::endl;
    for (int y = 0; y < 8; ++y) {
        for (int x = 0; x < 8; ++x) {
            Color c = reconstructedImage.getPixel(x, y);
            std::cout << "(" << (int)c.r << "," << (int)c.g << "," << (int)c.b << ") ";
        }
        std::cout << std::endl;
    }

    return 0;
}
